/**
 * @brief Atomic signal implementation.
 *
 * Define **STA_ATOMIC_ENABLE** to enable module.
 */
#ifndef STA_ATOMIC_SIGNAL_HPP
#define STA_ATOMIC_SIGNAL_HPP

#include <sta/config.hpp>
#ifdef STA_ATOMIC_ENABLE

#include <sta/signal.hpp>

#include <atomic>


namespace sta
{
	/**
	 * @brief Implementation of `Signal` interface using `std::atomic`.
	 */
	class AtomicSignal : public Signal
	{
	public:
		AtomicSignal();

		void notify() override;
		bool peek() override;
		bool test() override;
		void wait() override;

	private:
		std::atomic<bool> signal_;  /**< Atomic bool used as signal */
	};
} // namespace sta


#endif // STA_ATOMIC_ENABLE

#endif // STA_ATOMIC_SIGNAL_HPP
