/**
 * @brief Wrapper for HAL GPIO pins.
 *
 * Define **STA_HAL_GPIO_ENABLE** in `<sta/config.hpp>` to enable module.
 */
#ifndef STA_HAL_GPIO_PIN_HPP
#define STA_HAL_GPIO_PIN_HPP

#include <sta/config.hpp>
#ifdef STA_HAL_GPIO_ENABLE

#include <sta/gpio_pin.hpp>

#include <main.h>


namespace sta
{
	/**
	 * @brief Container for HAL GPIO Pin objects.
	 */
	class HalGpioPin : public GpioPin
	{
	public:
		HalGpioPin(GPIO_TypeDef * port, uint16_t pin);

		void setState(GpioPinState state) override;

	private:
		GPIO_TypeDef * port_;   /**< GPIO port */
		uint16_t pin_;          /**< GPIO pin */
	};
}

/**
 * @brief Create HalGpioPin object from pin label.
 *
 * @param label     Pin label
 */
#define STA_HAL_GPIO_PIN(label) sta::HalGpioPin{label##_GPIO_Port, label##_Pin}


#endif // STA_HAL_GPIO_ENABLE

#endif // STA_HAL_GPIO_PIN_HPP
